import { set } from './set';
import { freeze } from './freeze';
import { createTree, DEFAULT_ComparatorFn } from '../internals';
export function fromPairs(pairs, ComparatorFn = DEFAULT_ComparatorFn) {
    const tree = createTree(true, ComparatorFn);
    var pair;
    if (Array.isArray(pairs)) {
        for (var i = 0; i < pairs.length; i++) {
            pair = pairs[i];
            set(pair[0], pair[1], tree);
        }
    }
    else {
        const it = pairs[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            pair = current.value;
            set(pair[0], pair[1], tree);
        }
    }
    return freeze(tree);
}
/**
 * Creates a new `RedBlackTree` from a plain input object. If no ComparatorFn function is supplied, keys are compared
 * using logical less-than and greater-than operations, which will generally only be suitable for numeric or string keys.
 *
 * @export
 * @template V The type of values in the tree
 * @param {Associative<V>} obj The input object from which to create a new tree
 * @param {ComparatorFn<any>} [ComparatorFn] A comparison function, taking two keys, and returning a value less than 0 if the
 *                                     first key is smaller than the second, a value greater than 0 if the first key is
 *                                     greater than the second, or 0 if they're the same.
 * @returns {RedBlackTree<string, V>} A tree populated with the keys and values of the input object
 */
export function fromObject(obj, ComparatorFn) {
    const tree = createTree(true, ComparatorFn);
    const keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        set(key, obj[key], tree);
    }
    return freeze(tree);
}

//# sourceMappingURL=from.js.map
