import { isImmutable } from '@collectable/core';
import { createTree, findPath, findSuccessor, PathNode, NONE, isNone, editable, cloneAsMutable, doneMutating, swapNodeContents, swapNodeColors, editLeftChild, editRightChild, rotateLeft, rotateRight, updateCount, } from '../internals';
export function remove(key, tree) {
    if (tree._size === 0) {
        return tree;
    }
    var originalTree = tree;
    var immutable = isImmutable(tree._owner) && (tree = cloneAsMutable(tree), true);
    tree._root = editable(tree._group, tree._root);
    var p = findPath(key, tree._root, tree._compare, tree._group);
    if (p.next !== 0 /* NONE */) {
        PathNode.release(p, tree._root);
        return originalTree;
    }
    if (tree._size === 1) {
        PathNode.release(p, tree._root);
        if (immutable) {
            return createTree(false, tree._compare);
        }
        tree._root = NONE;
        return tree;
    }
    var current = p.node;
    var hasRight = !isNone(current._right);
    var hasLeft = !isNone(current._left);
    var debt = 0;
    if (hasRight || hasLeft) {
        if (hasRight) {
            if (hasLeft) {
                p = findSuccessor(tree._compare, p, tree._group);
                swapNodeContents(p.node, current);
                current = p.node;
            }
            debt = current._red ? 0 : 1;
            current = editRightChild(tree._group, current);
        }
        else if (hasLeft) {
            debt = current._red ? 0 : 1;
            current = editLeftChild(tree._group, current);
        }
    }
    else {
        debt = current._red ? 0 : 1;
        current = NONE;
    }
    p.replace(current);
    if (current._red && p.parent.node._red) {
        current._red = false;
        debt = 0;
    }
    if (debt > 0) {
        var parent = (p = p.release()).node, gp = p.parent, sibling = NONE, left = NONE, right = NONE;
        do {
            var branch = p.next;
            if (p.isNone()) {
                p.node._red = false;
                updateCount(p.node);
                debt--;
            }
            else if (branch === 1 /* LEFT */) {
                if ((sibling = editRightChild(tree._group, parent))._red) {
                    swapNodeColors(parent, sibling);
                    rotateLeft(gp, parent, sibling, tree);
                    p.node = sibling;
                    gp = p;
                    p = PathNode.next(parent, p, 1 /* LEFT */);
                }
                else if (!(left = sibling._left)._red && !sibling._right._red) {
                    sibling._red = true;
                    if (parent._red) {
                        parent._red = false;
                        debt--;
                        if (p.isNone()) {
                            tree._root = current;
                        }
                    }
                    else {
                        current = parent;
                        p = p.release();
                        gp = p.parent;
                        parent = p.node;
                    }
                    updateCount(parent);
                }
                else {
                    if (!sibling._right._red && left._red) {
                        left = editLeftChild(tree._group, sibling);
                        p.next = 2 /* RIGHT */;
                        swapNodeColors(sibling, left);
                        rotateRight(p, sibling, left, tree);
                        sibling = left;
                    }
                    right = editRightChild(tree._group, sibling);
                    rotateLeft(gp, parent, sibling, tree);
                    sibling._red = parent._red;
                    parent._red = false;
                    right._red = false;
                    debt--;
                }
            }
            else {
                if ((sibling = editLeftChild(tree._group, parent))._red) {
                    rotateRight(gp, parent, sibling, tree);
                    swapNodeColors(parent, sibling);
                    p.node = sibling;
                    gp = p;
                    p = PathNode.next(parent, p, 2 /* RIGHT */);
                }
                else if (!(right = sibling._right)._red && !sibling._left._red) {
                    sibling._red = true;
                    if (parent._red) {
                        parent._red = false;
                        debt--;
                        if (p.isNone()) {
                            tree._root = current;
                        }
                    }
                    else {
                        current = parent;
                        p = p.release();
                        gp = p.parent;
                        parent = p.node;
                    }
                    updateCount(parent);
                }
                else {
                    if (!sibling._left._red && right._red) {
                        right = editRightChild(tree._group, sibling);
                        p.next = 1 /* LEFT */;
                        swapNodeColors(sibling, right);
                        rotateLeft(p, sibling, right, tree);
                        sibling = right;
                    }
                    left = editLeftChild(tree._group, sibling);
                    rotateRight(gp, parent, sibling, tree);
                    sibling._red = parent._red;
                    parent._red = false;
                    left._red = false;
                    debt--;
                }
            }
        } while (debt > 0);
    }
    if (p.isActive()) {
        current = PathNode.releaseAndRecount(p, current);
    }
    tree._size--;
    return immutable ? doneMutating(tree) : tree;
}

//# sourceMappingURL=remove.js.map
